#!/usr/bin/env python
#prog16wd2.py
#fuer Raspberry Pi

#Wesentliche Hinweise zur ISP-Programmierung
#von Atmel-Controllern finden sich neben 
#den Angaben in den Datenblaettern der betreffenden
#Mikrocontroller.
#Die "Page Size" der AVR-Controller, die mit diesem Python-Programm programmiert werden #koennen, ist nach den Angaben in den Datenblaettern 16 words.
#Dies trifft u. a. zu fuer ATtiny13, ATtiny24, ATtiny25, ATtiny2313, ATTiny26, #ATtiny261 

#H. Nieder, Fruehjahr 2016, geringfuegig ergaenzt im Mai 2020

import sys
import time
import RPi.GPIO as GPIO


#GPIO-Pins
MOSI=8
SCK =7
MISO=25
RES=23

#GPIO-Festlegungen
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

GPIO.setup(MOSI, GPIO.OUT)
GPIO.setup(MISO, GPIO.IN)
GPIO.setup(SCK, GPIO.OUT)
GPIO.setup(RES, GPIO.OUT)

#"Umbennungen" fuer GPIO.output etc.  
out=GPIO.output
inp=GPIO.input
low=GPIO.LOW 
hi=GPIO.HIGH


#Anfangszustand
out(RES,low)
Byte=0
z=0
k=0
wordzahl=0
zahl2er=0
page_r=0

Byte_pos=0
n=0
#Liste wird gefuellt mit den Bytes 0xFF/255
prog_lst=[]
for Byte_pos in range(0,2050):
    prog_lst.insert(Byte_pos,255)

def SPI_aus(Byte):
    #Byte=255-Byte 
    Bit7_wt=128
    n=0
    for n in range(0,8):
        if (Byte & Bit7_wt) >0:
            out(MOSI,low)      
        else:
            out(MOSI,hi)
        time.sleep(0.0002)
        out(SCK,low)
        Bit7_wt=Bit7_wt>>1#1 Bit nach rechts schieben        
        time.sleep(0.0002) 
        out(SCK,hi) 
        time.sleep(0.0002)   
    out(MOSI,hi)

#Umwandlung Hex--> Dez, hier mit Dictionary
DEZ_dt={"0":0,"1":1,"2":2,"3":3,"4":4,"5":5,"6":6,"7":7,"8":8,"9":9,"A":10,"B":11,"C":12,"D":13,"E":14,"F":15}
def HexToInt(Hexzahl):
    H=0
    L=0
    Hnib=Hexzahl[0:1]  
    Lnib=Hexzahl[1:2]
    Ergebnis=16*DEZ_dt[Hnib]+DEZ_dt[Lnib]    
    Byte=Ergebnis
    return Byte

def prog_en():
    SPI_aus(0xAC)
    SPI_aus(0x53)
    SPI_aus(0)
    SPI_aus(0)    

def erase():
    SPI_aus(0xAC)
    SPI_aus(0x80)
    SPI_aus(0)
    SPI_aus(0)

r=130
anzahl=0
wordzahl=0
zahl=0
zahl2er=0
zahl1er=0
page_r=0
adres_hi=0
adres_lo=0
k=0
z=0
address=0

def load_prog_mem_page_R1():
    global z    
    #dat_zeile=dat_str
    for p in range(0,8):
        SPI_aus(0x40)
        SPI_aus(adres_hi)
        SPI_aus(p)        
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2
        Byte=prog_lst[address+k]
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)       
        SPI_aus(0x48)
        SPI_aus(adres_hi)
        SPI_aus(p)
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2+1
        Byte=prog_lst[address+k]
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)        

def load_prog_mem_page_R0_R():
    global z    
    #dat_zeile=dat_str
    for p in range(0,page_r2):
        SPI_aus(0x40)
        SPI_aus(adres_hi)
        SPI_aus(p)        
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2
        Byte=prog_lst[address+k]        
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)
        SPI_aus(0x48)
        SPI_aus(adres_hi)
        SPI_aus(p)
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2+1
        Byte=prog_lst[address+k]
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)    

def load_prog_mem_page_R2():
    global z
    #dat_zeile=dat_str
    for p in range(8,16):
        SPI_aus(0x40)
        SPI_aus(adres_hi)
        SPI_aus(p)        
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2
        Byte=prog_lst[address+k]        
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)
        SPI_aus(0x48)
        SPI_aus(adres_hi)
        SPI_aus(p)
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2+1
        Byte=prog_lst[address+k]
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)    

def load_prog_mem_page_R1_R():
    global z
    #dat_zeile=dat_str
    for p in range(8,8+page_r2):
        SPI_aus(0x40)
        SPI_aus(adres_hi)
        SPI_aus(p)        
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2
        Byte=prog_lst[address+k]
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)        
        SPI_aus(0x48)
        SPI_aus(adres_hi)
        SPI_aus(p)
        #Adresse des Bytes in der Liste
        address=adres_hi*256+p*2+1
        Byte=prog_lst[address+k]
        #print"Byte:"+("{0:3X}".format(Byte))
        SPI_aus(Byte)


def anzahl_lesen():
    global page_z
    global page_r
    global page_r2
    global page_r1    
    global dat_str
    zL=0
    #Anzahl der Zeilen bestimmen  
    hexdatei=open(hex_datei)
    dat_zeile=hexdatei.readline()
    while dat_zeile:
        zL=zL+1
        dat_zeile=hexdatei.readline()
    hexdatei.close()
    #Anzahl=zL-1, also ohne Zeile mit ":00000001FF"
    hexdatei=open(hex_datei)
    for r in range(0,zL-1):    
        dat_zeile=hexdatei.readline()
        byte_z_s=dat_zeile[1:3]
        byte_z=HexToInt(byte_z_s)
        adr_hi_s=dat_zeile[3:5]
        adr_hi=HexToInt(adr_hi_s)
        adr_lo_s=dat_zeile[5:7]
        adr_lo=HexToInt(adr_lo_s)
        adr=adr_hi*256+adr_lo
              
        for pos in range(0,byte_z):
            byt_asc=dat_zeile[9+pos*2:11+pos*2]
            byt_wt=HexToInt(byt_asc)
            prog_lst[adr+pos]=byt_wt
  
    anzahl=(adr+pos+1) 
    print "Words: ",anzahl/2
    page_z=anzahl/32
    print "Vollst. Pages/page_z: ",page_z
    page_r=anzahl%32
    #print "Restl. Bytes/page_r: ",page_r
    page_r1=page_r/16
    page_r2=(page_r%16)/2
    #print "Vollst. halbe Pages: ",page_r1
    #print "Restl.Words/page_r2: ",page_r2
    hexdatei.close() 
    time.sleep(0.2) 

adr=0
ad_hi=0
ad_lo=0
n=0

def Block16p_w():
    global z 
    global k
    k=0
    z=0
    global n
    global page_z
    for n in range(0,page_z):  # 
        load_prog_mem_page_R1()
        load_prog_mem_page_R2() 
        k=k+32              
        SPI_aus(0x4C)
        adr= n*16
        ad_hi=adr/256
        ad_lo=adr%256
        SPI_aus(ad_hi) 
        #print "ad_hi: ",hex(ad_hi)
        SPI_aus(ad_lo)
        #print "ad_lo: ",hex(ad_lo)
        SPI_aus(0x00)
        print"Page "+str(n+1)+" von "+str(page_z)      
        z=(n+1)*64
    #print"n**: ",str(n)
    #print

def Block16R_w():
    global z
    global n
    SPI_aus(0x4C)
    adr=n*16#
    ad_hi=adr/256
    ad_lo=adr%256
    SPI_aus(ad_hi)  #war(page_hi[k])
    #print "ad_hi: ",hex(ad_hi)
    SPI_aus(ad_lo)
    #print "ad_lo: ",hex(ad_lo)
    SPI_aus(0x00)   
    z=z+4*page_r2

print
hex_datei=raw_input("HEX-Datei: ")
try:
    hexdatei=open(hex_datei)
except:
    print"Fehler im Dateinamen"
    sys.exit(0)

print
anzahl_lesen()
hexdatei.close()
print

out(RES,hi)
time.sleep(0.02)
prog_en()
time.sleep(0.02)
erase()
time.sleep(0.02)

#Wenn vollstaendige Zweierreihen
if page_z>=1:
    Block16p_w() 
    #print"n*: ",str(n)    
#
    if page_r1==0:
        if page_r2>0:                 
            #Block_r2_write_lo()
            load_prog_mem_page_R0_R()   
        n=n+1#k=k+1
        #print"n:", str(n)
        #print
        Block16R_w()
# 
    if page_r1==1: 
          
        #Block_r1_write()
        load_prog_mem_page_R1()
        z=z+32  
        if page_r2>0:            
            #Block_r2_write_hi()
            load_prog_mem_page_R1_R()       
        n=n+1#k=k+1  
        #print"n: ",str(n) 
        #print         
        Block16R_w()
#
#Wenn keine vollstaendige Zweierreihen
if page_z==0:
    k=0
    n=0
#
    if page_r1==0:        
        if page_r2>0:                  
            #Block_r2_write_lo()
            load_prog_mem_page_R0_R()   
        #k=k+1
        Block16R_w()
#    
    if page_r1==1:        
        #Block_r1_write()
        load_prog_mem_page_R1()
        z=z+32  
        if page_r2>0:
            #print"page_r2_pg_z=0: ",page_r2
            #Block_r2_write_hi()
            load_prog_mem_page_R1_R()       
        #k=k+1          
        Block16R_w()
#
hexdatei.close()
print
print "Programmierung beendet"
print
#print str(z/4)+" Words programmiert"
print 
time.sleep(1)

out(MOSI,low)
out(SCK,low)
time.sleep(1)
out(RES,low)

